# GO-MASTIFF MODEL

## Overview

The **GO-MASTIFF** model is a reimplementation of the [**MASTIFF**](https://git.ufz.de/oesa/mastiff) (**M**ultiple **A**quatic **ST**ressors **I**n **F**lowing **F**ood webs) model in the Go (Golang) programming language. It is a spatially explicit simulation tool designed for analyzing multi-stressed riverine ecosystems. It focuses on understanding the impact of organismic drift and dispersal processes on:

1. The community structure of benthic grazers.
2. The effectiveness of grazer-mediated control of eutrophication.
3. The resilience against local stressors across different river segments.

## Translation from Java

The original MASTIFF model was developed using Java. This Go version preserves the core functionality of the initial implementation and is based on the ODD-protocol (Version: 11.05.2021 by Laura Meier) while adopting Go's modern programming style. The architecture has transitioned from an object-oriented approach to an Entity Component System (ECS) framework utilizing [Ark](https://github.com/mlange-42/ark) and enhancing modularity and performance. The main packages are params (parameters), model (state variables) and sys (systems).

## Scheduling

The model is divided into different systems, which contain all the methods. For every tick (equivalent to one week), all systems are updated in the following order:

- Development
- Mortality
- Reproduction
- BiofilmGrowth
- Eutrophication
- (Grazing)
- (Drift)

## How to Use

To get started with the GO-MASTIFF model:

1. **Install Go:**

    Ensure that you have Go installed on your system. You can download it from the official [Go website](https://golang.org/dl/).

2. **Clone the Repository:**  

    Open your terminal and clone the repository by executing the following command:

    ```bash
    git clone https://git.ufz.de/oesa/go-mastiff.git
    ```

3. **Run modell**  

    Navigate to the cloned repository folder and run the model by entering:

    ```bash
    go run .
    ```

    This will run the model with a default parameterization.
