package main

import (
	"encoding/json"
	"log"
	"os"

	"git.ufz.de/oesa/go-mastiff/model"
	"git.ufz.de/oesa/go-mastiff/params"
	"git.ufz.de/oesa/go-mastiff/sys"
	"github.com/mlange-42/ark-tools/app"
	"github.com/mlange-42/ark-tools/system"
	"github.com/mlange-42/ark/ecs"
)

func main() {
	var paramSet params.ParameterSet
	var m model.Model

	switch len(os.Args) {
	case 2:
		// When exactly one additional argument exists
		segNumber := getNumberOfSegments(os.Args[1])
		paramSet = params.NewParameterSet(segNumber)
		readParamJSON(os.Args[1], &paramSet)

		m = model.New(segNumber)
	case 1:
		// When no additional arguments are provided, use default parameterization
		paramSet = params.NewParameterSet(model.DefaultSegNumber)

		m = model.New(model.DefaultSegNumber)
	default:
		// More than one argument: print an error and exit
		panic("Too many arguments")
	}

	app := app.New()

	ecs.AddResource(&app.World, &paramSet)
	ecs.AddResource(&app.World, &m)

	app.AddSystem(&sys.Development{})
	app.AddSystem(&sys.Mortality{})
	app.AddSystem(&sys.Reproduction{})
	app.AddSystem(&sys.BiofilmGrowth{})
	app.AddSystem(&sys.Eutrophication{})

	app.AddSystem(&system.FixedTermination{Steps: 100})

	app.Run()
}

func readParamJSON(path string, paramSet *params.ParameterSet) {
	file, err := os.Open(path)
	if err != nil {
		log.Fatalf("Unable to open parameters file: %v", err)
	}
	defer file.Close()

	decoder := json.NewDecoder(file)
	decoder.DisallowUnknownFields()

	if err := decoder.Decode(paramSet); err != nil {
		log.Fatalf("Unable to decode JSON: %v", err)
	}
}

func getNumberOfSegments(path string) int {
	file, err := os.Open(path)
	if err != nil {
		log.Fatalf("Unable to open parameters file: %v", err)
	}
	defer file.Close()

	var data params.ParameterSet
	if err := json.NewDecoder(file).Decode(&data); err != nil {
		log.Fatalf("Unable to decode JSON: %v", err)
	}

	// If NumberOfSegments is not set (or is zero), use the default value
	if data.NumberOfSegments == 0 {
		return model.DefaultSegNumber
	}

	return data.NumberOfSegments
}
