package main

import (
	"os"
	"path/filepath"
	"testing"

	"git.ufz.de/oesa/go-mastiff/params"
	"github.com/stretchr/testify/assert"
)

const testJSONContent = `{
	"NumberOfSegments": 20
  }`

func TestGetNumberOfSegments(t *testing.T) {
	// Create a temporary directory
	tempDir := t.TempDir()

	// Define the path for the temporary JSON file
	tempFilePath := filepath.Join(tempDir, "test_parameters.json")

	// Create and write to the temporary JSON file
	if err := os.WriteFile(tempFilePath, []byte(testJSONContent), 0644); err != nil {
		t.Fatal(err)
	}

	// Use this temporary file for the test
	numSegments := getNumberOfSegments(tempFilePath)

	assert.Equal(t, 20, numSegments)
}

func TestReadParams(t *testing.T) {
	// Create a temporary directory
	tempDir := t.TempDir()

	// Define the path for the temporary JSON file
	tempFilePath := filepath.Join(tempDir, "test_parameters.json")

	// Create and write to the temporary JSON file
	if err := os.WriteFile(tempFilePath, []byte(testJSONContent), 0644); err != nil {
		t.Fatal(err)
	}

	// Use this temporary file for the test
	var paramSet params.ParameterSet
	readParamJSON(tempFilePath, &paramSet)

	assert.Equal(t, 20, paramSet.NumberOfSegments)
}
