package model

import (
	"git.ufz.de/oesa/go-mastiff/params"
)

// Segment represents a segment in the model.
type Segment struct {
	SegmentID                     int       // Segment ID
	SpeciesSet                    []Species // Species state variables
	BaseBiofilm                   float64   // BiofilmF in Java
	ThickSusceptibleBiofilm       float64   // BiofilmN in Java
	MicroNicheBiofilm             float64   // BiofilmM in Java
	BaseBiofilmForage             float64   // Forageable base biofilm
	ThickSusceptibleBiofilmForage float64   // Forageable thick biofilm
	MicroNicheBiofilmForage       float64   // Forageable micro niche biofilm
	RippedBaseBiofilm             float64   // Base biofilm ripped off at the last ripp off event
	RippedThickSusceptibleBiofilm float64   // Thick biofilm ripped off at the last ripp off event
	IsBaseEutrophic               bool      // True if base biofilm segment is eutrophicated
	IsThickSusceptibleEutrophic   bool      // True if thick susceptible biofilm compartment of the segment is eutrophicated
	DeathTickFila                 int64     // Life span of the filamentous algae
}

// NewSegment creates a new Segment with the specified segment ID.
func NewSegment(segNumber int) Segment {
	return Segment{
		SegmentID: segNumber,
		SpeciesSet: []Species{
			{
				ID:         params.Limpet,
				Abundance:  []float64{20000, 20000, 20000},
				Immigrants: []float64{2000, 2000, 2000},
			},
			{
				ID:         params.Mayfly,
				Abundance:  []float64{10000, 10000, 10000},
				Immigrants: []float64{1000, 1000, 1000},
			},
			{
				ID:         params.Chironomid,
				Abundance:  []float64{100000, 100000, 100000},
				Immigrants: []float64{10000, 10000, 10000},
			},
		},
		BaseBiofilm:                 10000.0,
		ThickSusceptibleBiofilm:     10000.0,
		MicroNicheBiofilm:           10000.0,
		IsBaseEutrophic:             false,
		IsThickSusceptibleEutrophic: false,
		DeathTickFila:               -1,
	}
}
