package params

// ParameterSet contains the parameters for the model, including model parameters, species parameters, and segment parameters.
type ParameterSet struct {
	Mod              ModelParams     // Model parameters
	Spec             []SpeciesParams // Species parameters
	Seg              []SegmentParams // Segment parameters
	NumberOfSegments int             // Number of segments
}

// NewParameterSet creates a new ParameterSet with the specified number of segments and initializes its fields.
func NewParameterSet(segNumber int) ParameterSet {
	segments := make([]SegmentParams, segNumber)
	for i := range segments {
		segments[i] = newSegmentParams(i)
	}

	return ParameterSet{
		Mod: newModelParams(),
		Spec: []SpeciesParams{
			newSpeciesParameters(Limpet),
			newSpeciesParameters(Mayfly),
			newSpeciesParameters(Chironomid),
		},
		Seg:              segments,
		NumberOfSegments: segNumber,
	}
}
