package sys

import (
	"testing"

	"git.ufz.de/oesa/go-mastiff/model"
	"git.ufz.de/oesa/go-mastiff/params"
	"github.com/mlange-42/ark-tools/app"
	"github.com/mlange-42/ark-tools/system"
	"github.com/mlange-42/ark/ecs"
	"github.com/stretchr/testify/assert"
)

// TestBiofilmSystem makes a simple test run of the BiofilmGrowth system
func TestBiofilmSystem(t *testing.T) {
	app := app.New()
	m := model.New(model.DefaultSegNumber)
	paramSet := params.NewParameterSet(model.DefaultSegNumber)

	paramSet.Mod.Biofilm.MortalityRate = 0.0
	paramSet.Mod.Biofilm.CoreCapacity = 10000.0
	paramSet.Mod.Biofilm.CapacityMicroNiche = 1000.0
	paramSet.Mod.Biofilm.FractionBase = 0.4
	paramSet.Mod.Biofilm.FractionThickSusceptible = 0.4
	paramSet.Mod.Biofilm.FractionMicroNiche = 0.2

	ecs.AddResource(&app.World, &paramSet)
	ecs.AddResource(&app.World, &m)

	app.AddSystem(&BiofilmGrowth{})
	app.AddSystem(&system.FixedTermination{Steps: 96})

	app.Run()

	// Base and ThickPossible biofilm
	assert.Equal(t, 5000, int(m.Segments[0].BaseBiofilm))
	assert.Equal(t, 5000, int(m.Segments[0].ThickSusceptibleBiofilm))
	assert.Equal(t, 1000, int(m.Segments[0].MicroNicheBiofilm))
}
