package sys

import (
	"testing"

	"git.ufz.de/oesa/go-mastiff/model"
	"git.ufz.de/oesa/go-mastiff/params"
	"github.com/mlange-42/ark-tools/app"
	"github.com/mlange-42/ark-tools/system"
	"github.com/mlange-42/ark/ecs"
	"github.com/stretchr/testify/assert"
)

func TestDevelopment(t *testing.T) {
	app := app.New()
	m := model.New(model.DefaultSegNumber)
	paramSet := params.NewParameterSet(model.DefaultSegNumber)

	ecs.AddResource(&app.World, &paramSet)
	ecs.AddResource(&app.World, &m)

	app.AddSystem(&Development{})
	app.AddSystem(&system.FixedTermination{Steps: 48})

	app.Run()

	// All individuals hatch during the year
	assert.Equal(t, 0, int(m.Segments[0].SpeciesSet[params.Limpet].Abundance[model.Egg]))
	assert.Equal(t, 0, int(m.Segments[0].SpeciesSet[params.Mayfly].Abundance[model.Egg]))
	assert.Equal(t, 0, int(m.Segments[0].SpeciesSet[params.Chironomid].Abundance[model.Egg]))

	// Limpets and mayflies hatch (month == 6) after the larvae become mature (month == 4), so not all individuals become mature after a year
	assert.Equal(t, 20000, int(m.Segments[0].SpeciesSet[params.Limpet].Abundance[model.Larvae]))
	assert.Equal(t, 10000, int(m.Segments[0].SpeciesSet[params.Mayfly].Abundance[model.Larvae]))
	assert.Equal(t, 0, int(m.Segments[0].SpeciesSet[params.Chironomid].Abundance[model.Larvae]))

	assert.Equal(t, 40000, int(m.Segments[0].SpeciesSet[params.Limpet].Abundance[model.Mature]))
	assert.Equal(t, 20000, int(m.Segments[0].SpeciesSet[params.Mayfly].Abundance[model.Mature]))
	assert.Equal(t, 300000, int(m.Segments[0].SpeciesSet[params.Chironomid].Abundance[model.Mature]))
}
