package sys

import (
	"testing"

	"git.ufz.de/oesa/go-mastiff/model"
	"git.ufz.de/oesa/go-mastiff/params"
	"github.com/mlange-42/ark-tools/app"
	"github.com/mlange-42/ark-tools/system"
	"github.com/mlange-42/ark/ecs"
	"github.com/stretchr/testify/assert"
)

func TestReproduction(t *testing.T) {
	app := app.New()
	paramSet := params.NewParameterSet(model.DefaultSegNumber)
	m := model.New(model.DefaultSegNumber)

	paramSet.Spec[params.Limpet].Offspring = 10
	paramSet.Spec[params.Mayfly].Offspring = 10
	paramSet.Spec[params.Chironomid].Offspring = 10

	paramSet.Spec[params.Mayfly].Dispersal = params.DispersalParameters{
		ProbStay:     0,
		ProbUp:       0,
		ProbDown:     1,
		DistMeanUp:   1,
		DistMeanDown: 1,
	}
	paramSet.Spec[params.Chironomid].Dispersal = params.DispersalParameters{
		ProbStay:     0,
		ProbUp:       1,
		ProbDown:     0,
		DistMeanUp:   1,
		DistMeanDown: 1,
	}

	// Setting parameters
	ecs.AddResource(&app.World, &m)
	ecs.AddResource(&app.World, &paramSet)
	app.AddSystem(&Reproduction{})

	app.AddSystem(&system.FixedTermination{Steps: 48})
	app.Run()

	// Limpets just reproduce with 10 eggs per mature individual
	assert.Equal(t, []float64{220000, 20000, 20000}, m.Segments[0].SpeciesSet[params.Limpet].Abundance)

	// Mayfly disperse downstream
	assert.Equal(t, []float64{10000, 10000, 0}, m.Segments[0].SpeciesSet[params.Mayfly].Abundance)

	// Chironomids disperse upstream
	assert.Equal(t, []float64{100000, 100000, 0}, m.Segments[9].SpeciesSet[params.Chironomid].Abundance)
}
